package com.dcsoft.system.uniubi.service;


import com.dcsoft.system.domain.SysEquipment;
import com.dcsoft.system.domain.SysPeople;
import com.dcsoft.system.domain.SysRule;
import com.dcsoft.system.visitor.domain.Visitor;


import java.util.List;

/**
 * Uface 服务层
 *
 * @author xueyi
 */
public interface ISysApiService {


    //获取token值
    String getToken();

    //人员接口-创建识别对象的主体信息
    String admitCreate(SysPeople people);

    //人员接口-更新识别对象的主体信息
    String admitUpdate(SysPeople people);

    //人员接口-删除识别对象的主体信息
    String admitDelete(String admitGuids);

    //人员接口-上传人像照片
    String faceRegister(SysPeople people);

    //人员接口-删除人像照片
    String faceDelete(SysPeople people);

    //设备信息-设备添加
    String deviceCreate(SysEquipment equipment);

    //设备信息-设备修改
    String deviceUpdate(SysEquipment equipment);

    //设备信息-删除设备信息
    String deviceDelete(SysEquipment equipment);

    //设备信息-设备配置查询
    String deviceSettingQuery(SysEquipment equipment);

    //设备信息-远程配置设备参数
    String deviceSettingUpdate(SysEquipment equipment);

    //设备信息-批量查询设备在线状态
    String deviceOnlineState(List<SysEquipment> equipment);

    //设备信息-设备状态操作的指令下发
    String deviceCommand(SysEquipment equipment,int operateType);

    String deviceInteractive(SysEquipment equipment);

    //人像门禁服务-设备授权识别主体
    String authDevice(SysRule rule, SysPeople people);

    String authDevice(SysRule rule, SysPeople people,SysEquipment equipment);

    //人像门禁服务-设备修改识别主体权限
    String authDeviceUpdate(SysRule rule,SysPeople people);

    //人像门禁服务-设备销权识别主体
    String authDeviceRevoke(SysEquipment equipment, SysPeople people);

    //人像门禁服务-授权列表查询
    String sceneAuth();

    //访客人员接口-创建识别对象的主体信息
    String admitCreate(Visitor visitor);

    //访客人员接口-更新识别对象的主体信息
    String admitUpdate(Visitor visitor);

    //访客人员接口-上传人像照片
    String faceRegister(Visitor visitor);

    //访客人员接口-删除人像照片
    String faceDelete(Visitor visitor);

    //访客人像门禁服务-设备授权识别主体
    String authDeviceExamine(Visitor visitor);

    String authDeviceRevoke(Visitor visitor);


}
